@@ -1,7 +1,19 @@
-# Songwrite
-# Copyright (C) 2001-2002 Jean-Baptiste LAMY
+# SongWrite
+# Copyright (C) 2001-2004 Jean-Baptiste LAMY -- jibalamy@free.fr
 #
-# This program is free software. See README or LICENSE for the license terms.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from __future__ import nested_scopes
 
@@ -88,6 +100,7 @@ class App(Toplevel):
     export_menu.add_command(label = _("Lilypond (no lyrics)..."), command = self.on_export_lily)
     export_menu.add_command(label = _("LaTeX+Lilypond...")      , command = self.on_export_latex)
     export_menu.add_command(label = _("Postscipt...")           , command = self.on_export_ps)
+#    export_menu.add_command(label = _("AbcTab...")              , command = self.on_export_abctab)
     file_menu.add_cascade  (label = _("Export"), menu = export_menu)
     
     file_menu.add_command(label = _("preview_print")     , command = self.on_preview_print)
@@ -274,6 +287,7 @@ class App(Toplevel):
     self.width       = self.height = 0
     self.bind("<Expose>"  , self.on_expose)
     self.wm_protocol("WM_DELETE_WINDOW", self.on_close)
+    self.wm_command("songwrite")
     
     self.song = None
     
@@ -520,6 +534,7 @@ class App(Toplevel):
   def on_save_as(self, event = None):
     filename = self.prompt_filename(defaultextension = ".sw.xml", filetypes = ((_("XML files"), "*.sw.xml"),))
     if not filename: return
+    if not filename.endswith(".sw.xml"): filename += ".sw.xml"
     globdef.config.add_previous_file(filename)
     self.filename = self.song.filename = filename
     self.on_save()
@@ -545,7 +560,9 @@ class App(Toplevel):
     if last_cancelable and (not self.last_cancelable is last_cancelable):
       # The last cancelable action has changed => the song need to save.
       import tkMessageBox
-      if not tkMessageBox.askyesno(_("Confirmation"), _("The current song has been changed. Continue ?")):
+      x = tkMessageBox._show(_("Confirmation"), _("The current song has been changed. Continue ?"), tkMessageBox.QUESTION, tkMessageBox.YESNO)
+      if (x != "yes") and (not x is True):
+      #if not tkMessageBox.askyesno(_("Confirmation"), _("The current song has been changed. Continue ?")):
         return 1
       
     self.cancel = cancel.Context()
@@ -617,16 +634,20 @@ class App(Toplevel):
     except: self._treat_lily_error()
     
   def _treat_lily_error(self):
-    error, args, trace = sys.exc_info()
-    sys.excepthook(error, args, trace)
-    
-    partition, time = args
-    partition.view.select_at(time)
+    error_class, error, trace = sys.exc_info()
+    sys.excepthook(error_class, error, trace)
+    print
     
     import tkMessageBox
-    if   error is song.TimeError: tkMessageBox.showerror(_("Error while exporting!"), _("__time-error__"))
-    elif error is     ValueError: tkMessageBox.showerror(_("Error while exporting!"), _("__value-error__"))
-      
+    if isinstance(error, song.SongwriteError):
+      tkMessageBox.showerror(_(error.__class__.__name__), unicode(error))
+      if isinstance(error, song.TimeError):
+        if   error.note: error.partition.view.select_at(error.note.time, getattr(error.note, "stringid", 0))
+        elif error.partition and error.time: error.partition.view.select_at(error.time)
+        elif error.time and self.partition: self.partition.view.select_at(error.time)
+    else:
+      tkMessageBox.showerror(_(error.__class__.__name__), _("__UnknownError__"))
+        
   def on_export_latex(self):
     import latex
     
@@ -643,6 +664,12 @@ class App(Toplevel):
     try: latex.export2ps(self.song, filename)
     except: self._treat_lily_error()
     
+#   def on_export_abctab(self):
+#     import abctab
+#     reload(abctab)
+    
+#     print abctab.abctab(self.song)
+    
   def on_preview_print(self, event = None):
     import latex
     
@@ -1079,7 +1106,7 @@ class App(Toplevel):
       
   def on_insert_times(self):
     import tkSimpleDialog
-    nb_time = tkSimpleDialog.askinteger(_("Insert times..."), _("Insert how many times? (negative for removing)"))
+    nb_time = tkSimpleDialog.askinteger(_("Insert times..."), _("Insert how many times? (negative for removing)"), parent = self)
     if not nb_time: return
     at = self.selected_notes()[0].note.time
     delta = nb_time * 96
@@ -1119,7 +1146,7 @@ class App(Toplevel):
     
   def on_insert_mesures(self):
     import tkSimpleDialog
-    nb_mesure = tkSimpleDialog.askinteger(_("Insert bars..."), _("Insert how many bars? (negative for removing)"))
+    nb_mesure = tkSimpleDialog.askinteger(_("Insert bars..."), _("Insert how many bars? (negative for removing)"), parent = self)
     if not nb_mesure: return
     mesure = self.song.mesure_at(self.selected_notes()[0].note.time)
     mesure_pos = self.song.mesures.index(mesure)
@@ -1196,7 +1223,7 @@ class App(Toplevel):
     
   def on_note_arrange(self):
     import tkSimpleDialog
-    fret = tkSimpleDialog.askinteger(_("Arrange at fret..."), _("Arrange selected notes at fret?"))
+    fret = tkSimpleDialog.askinteger(_("Arrange at fret..."), _("Arrange selected notes at fret?"), parent = self)
     
     self.cancel.push()
     
@@ -1354,7 +1381,17 @@ class App(Toplevel):
   def time_to_x_vec(self, time): return time * self.zoom
   def time_to_second(self, time): return float(time) / (self.song.mesures[0].tempo * 2)
   
-  def on_dump(self):  print self.song.__xml__().getvalue()
+  def on_dump(self):
+    print self.song.__xml__().getvalue()
+    
+    #import lilypond
+    #reload(lilypond)
+    
+    #lily = lilypond.lilypond(self.song)
+    #print lily
+    #print >> open("/home/jiba/src/test2.ly", "w"), lily
+    
+    
   def on_dump_clipboard(self): print `selection`
   def on_gc(self):
     import gc
