@@ -1,20 +1,42 @@
-# Songwrite
-# Copyright (C) 2001-2003 Jean-Baptiste LAMY
+# SongWrite
+# Copyright (C) 2001-2004 Jean-Baptiste LAMY -- jibalamy@free.fr
 #
-# This program is free software. See README or LICENSE for the license terms.
-
-# This module is now independent from the rest of Songwrite, and so can be
-# re-used in other (GPLed) program to generate midi-file !
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from __future__ import nested_scopes
 import sys, os, struct, bisect, math, types
 
 from xml.sax.saxutils import escape
 
-VERSION = "0.12"
+VERSION = "0.13"
 
-TimeError = "TimeError"
+class SongwriteError(StandardError):
+  pass
 
+class TimeError(SongwriteError):
+  def __init__(self, time, partition = None, note = None):
+    self.time      = time
+    self.partition = partition
+    self.note      = note
+    self.args      = time, partition, note
+  def __unicode__(self):
+    return _("__%s__" % self.__class__.__name__)
+    
+class UnsingablePartitionError(TimeError):
+  pass
+  
 class _XMLContext:
   def __init__(self):
     self._next_id = 0
@@ -30,7 +52,7 @@ class _XMLContext:
       return self._next_id
     return id
   
-
+  
 def note_label(value):
   """note_label(value) -> string -- Return the string (e.g. "C(2)") for the given note int value."""
   if value == -1: return "_"
@@ -446,6 +468,7 @@ class Partition(TemporalData):
     self.header     = ""
     self.muted      = 0
     self.volume     = 255
+    self.tonality   = "C"
     
   def __setstate__(self, state): # Compatibility with older format
     self.__dict__.update(state)
@@ -522,27 +545,35 @@ class Partition(TemporalData):
     return "\xff\x10%s\x01\x00%s" % (struct.pack(">b", len(self.view.strings) + 2),
                                      "".join(map(lambda string: struct.pack(">b", string.basenote), self.view.strings)))
   
-  def voices(self):
+  def voices(self, single_voice = 0):
     "Gets the list of voices (non-overlapping sequence of notes) in this partition."
-    voices = [[], []]
-    for note in self.notes:
-      if   self.is_in_chord(note): voices[0].append(note) # Chords are not bass !
-      elif self._clash_note_voice(note, voices[0]): voices[1].append(note)
-      elif self._clash_note_voice(note, voices[1]): voices[0].append(note)
-      else:
-        if   note.linked and note.linked_from: # Add with the other note !
-          voices[note.linked_from in voices[1]].append(note)
-        elif note.duration < 96: voices[0].append(note) # Too short for a bass !
+    if single_voice:
+      voices = [[]]
+      for note in self.notes:
+        if self.is_in_chord(note) or self._clash_note_voice(note, voices[0]):
+          raise UnsingablePartitionError(note.time, self, self._clash_note_voice(note, voices[0]))
+        voices[0].append(note)
+        
+    else:
+      voices = [[], []]
+      for note in self.notes:
+        if   self.is_in_chord(note): voices[0].append(note) # Chords are not bass !
+        elif self._clash_note_voice(note, voices[0]): voices[1].append(note)
+        elif self._clash_note_voice(note, voices[1]): voices[0].append(note)
         else:
-          clashings = self.notes_at(note)
-          if len(clashings) == 1:
-            if getattr(note, "stringid", 6) >= 3: voices[1].append(note)
-            else:                                 voices[0].append(note)
+          if   note.linked and note.linked_from: # Add with the other note !
+            voices[note.linked_from in voices[1]].append(note)
+          elif note.duration < 96: voices[0].append(note) # Too short for a bass !
           else:
-            clashings.sort(lambda a, b: cmp(a.value, b.value))
-            if note is clashings[0]: voices[1].append(note)
-            else:                    voices[0].append(note)
-          
+            clashings = self.notes_at(note)
+            if len(clashings) == 1:
+              if getattr(note, "stringid", 6) >= 3: voices[1].append(note)
+              else:                                 voices[0].append(note)
+            else:
+              clashings.sort(lambda a, b: cmp(a.value, b.value))
+              if note is clashings[0]: voices[1].append(note)
+              else:                    voices[0].append(note)
+
     return voices
   
   def __repr__(self):
@@ -559,7 +590,7 @@ class Partition(TemporalData):
   
   def _clash_note_voice(self, note, voice):
     for n in voice:
-      if note.clash(n): return 1 # not self.is_in_chord(note)
+      if note.clash(n): return n # not self.is_in_chord(note)
       
     return 0
   
